@@ -167,14 +167,22 @@ import configargparse
 #----------------------------------------------
 class QbiRunner():
     def __init__(self, description = None):
+        '''Initialise QbiRunner class, creating the ArgParser member and setting
+        generic input options
+
+        Parameters
+        ----------
+        description : _type_, optional
+            _description_, by default None
+        '''
         self.description_ = description
         self.parser = configargparse.ArgParser(description)
         self.set_generic_options()
     
     #---------------------------------------------
     def set_generic_options(self):
-        '''
-        Set-up the generic options for the runner as args in the parser
+        '''Define set of generic options that all function that use the runner
+        will inherit
         '''
         #Config file
         self.parser.add('--config', required=False, is_config_file=True,
@@ -192,6 +200,10 @@ class QbiRunner():
         #Overwrite flag
         self.parser.add('--overwrite', type=bool_option, nargs='?', default=True, const = True,
             help='Flag to allow overwriting previous output')
+
+        #Archive existing work
+        self.parser.add('--archive_dir', type=str, nargs='?', default=None,
+            help='Name to archive existing output folder. If set to timestamp, will append a timestamp to current output')
         
         #Name of program log
         self.parser.add('--program_log', type=str, nargs='?', default='_program_log',
@@ -223,15 +235,38 @@ class QbiRunner():
         
     #----------------------------------------------
     def parse_args(self, args = None):
+        '''Parse args from command-line and/or config file
+
+        Parameters
+        ----------
+        args : _type_, optional
+            _description_, by default None
+
+        Returns
+        -------
+        _type_
+            _description_
+        '''
         if args is None:
             args = sys.argv[1:]
         return self.parser.parse_args(args)
 
     #-----------------------------------------------
     def run(self, fun, args = None):
-        '''
-        Run the given function fun inside the runner, setting up an output
+        '''Run the given function fun inside the runner, setting up an output
         directory and program/audit logs
+
+        Parameters
+        ----------
+        fun : _type_
+            _description_
+        args : _type_, optional
+            _description_, by default None
+
+        Returns
+        -------
+        _type_
+            _description_
         '''
         #Parse args (if none set, this will use sys.argv)
         options = self.parse_args(args)
@@ -243,16 +278,18 @@ class QbiRunner():
         #Set up output folder and program log
         if options.output_dir is None:
             options.output_dir = fun.__name__ + '_output'
-
         options.output_dir = os.path.join(options.data_dir, options.output_dir)
         os.makedirs(options.output_dir, exist_ok = options.overwrite)
 
+        #Get list of files in output dir for archiving
+        options.output_list = os.listdir(options.output_dir)
+
         #Get function args from options namespace
         args = get_function_args(fun, options)
 
         #If user set no log, then just run the function
         if options.no_log:
-            success = run_catch(args, fun)
+            success = run_catch(options, args, fun)
         else:
             success = run_with_logging(options, args, fun)
 
@@ -266,13 +303,32 @@ class QbiRunner():
         return exit_code
 
 #-----------------------------------------------
-def run_catch(args, fun):
-    '''
-    Trying running fun with input **args. If an exception is raised,
+def run_catch(options, args, fun):
+    '''Trying running fun with input **args. If an exception is raised,
     catch it, print it and return False.
 
     If the function runs to completion, return true.
+
+    _extended_summary_
+
+    Parameters
+    ----------
+    options : _type_
+        _description_
+    args : _type_
+        _description_
+    fun : _type_
+        _description_
+
+    Returns
+    -------
+    _type_
+        _description_
     '''
+    #Do archiving here, so it get logged correctly
+    archive_existing_output(
+        options.data_dir, options.archive_dir, options.output_dir, options.output_list)
+
     try:
         fun(**args)
         success = True
@@ -284,8 +340,7 @@ def run_catch(args, fun):
 
 #-----------------------------------------------
 def run_with_logging(options, args, fun):
-    '''
-    Run the function fun with inputs **args, creating program, config
+    '''Run the function fun with inputs **args, creating program, config
     and audit logs as specified by the options.
 
     If a program log is created (ie. options.no_log = False) then
@@ -294,6 +349,20 @@ def run_with_logging(options, args, fun):
     fun will be called via run_catch, so any exceptions raised will be
     caught and logged. This function will return True if fun ran to
     completion and False if an exception was caught
+
+    Parameters
+    ----------
+    options : _type_
+        _description_
+    args : _type_
+        _description_
+    fun : _type_
+        _description_
+
+    Returns
+    -------
+    _type_
+        _description_
     '''
     #Need to set-up logging info
 
@@ -323,7 +392,7 @@ def run_with_logging(options, args, fun):
         #Run the function within the context of stdout and stderr being
         #redirected to the program log
         with redirect_stderr(program_log), redirect_stdout(program_log):
-            success = run_catch(args, fun)
+            success = run_catch(options, args, fun)
 
         #Finalise the program log
         finalise_log(program_log, success)
@@ -335,9 +404,48 @@ def run_with_logging(options, args, fun):
     return success
 
 #-----------------------------------------------
-def initialise_audit_log(audit_log_path, program_log_path):
+def archive_existing_output(data_dir, archive_dir, output_dir, output_list):
+    '''Move any existing files and folders inside output_dir to an archive_dir
+
+    Parameters
+    ----------
+    data_dir : _type_
+        _description_
+    archive_dir : _type_
+        _description_
+    output_dir : _type_
+        _description_
+    output_list : _type_
+        _description_
     '''
-    Create a new audit log at the specified path
+    if archive_dir is None or not output_list:
+        return
+
+    if archive_dir == 'timestamp':
+        archive_dir = os.path.join(
+            output_dir, 
+            datetime.today().strftime('_archive%Y%m%d_%H%M%S'))
+    else:
+        archive_dir = os.path.join(data_dir, archive_dir)
+
+    print('Output folder exists and archiving is on: '
+            f'moving {data_dir} to {archive_dir}')
+    os.makedirs(archive_dir, exist_ok=False)
+    for obj_name in output_list:
+        if not obj_name.lower().startswith('_archive'):
+            os.rename(
+                os.path.join(output_dir, obj_name), 
+                os.path.join(archive_dir, obj_name))
+#-----------------------------------------------
+def initialise_audit_log(audit_log_path, program_log_path):
+    '''Create a new audit log at the specified path
+
+    Parameters
+    ----------
+    audit_log_path : _type_
+        _description_
+    program_log_path : _type_
+        _description_
     '''
     os.makedirs(os.path.dirname(audit_log_path), exist_ok = True)
     with open(audit_log_path, 'wt') as log:
@@ -346,18 +454,30 @@ def initialise_audit_log(audit_log_path, program_log_path):
 
 #-----------------------------------------------
 def finalise_audit_log(audit_log_path, success):
-    '''
-    Finalise the audit log
+    '''Finalise the audit log
+
+    Parameters
+    ----------
+    audit_log_path : _type_
+        _description_
+    success : _type_
+        _description_
     '''
     with open(audit_log_path, 'at') as log:
         finalise_log(log, success)
 
 #-----------------------------------------------
 def finalise_log(log, success):  
-    '''
-    Finalise an open audit/program log with file object log,
+    '''Finalise an open audit/program log with file object log,
     logging the time of completion and whether the function
     exited successfully of raised an exception
+
+    Parameters
+    ----------
+    log : _type_
+        _description_
+    success : _type_
+        _description_
     '''
     if success:
         print(f'{sys.argv[0]} completed successfully.', file=log)
@@ -369,9 +489,13 @@ def finalise_log(log, success):
 
 #-----------------------------------------------
 def initialise_log(log):
-    '''
-    Initialise an open audit/program log with file object log,
+    '''Initialise an open audit/program log with file object log,
     logging the start time, user, host and command-line args
+
+    Parameters
+    ----------
+    log : _type_
+        _description_
     '''
     date_str = datetime.today().strftime('%Y%m%d %H:%M:%S')
     print(f'Log opened at {date_str}', file=log)
@@ -382,10 +506,16 @@ def initialise_log(log):
 
 #-----------------------------------------------
 def write_config_log(options, config_log_path):
-    '''
-    Write the complete set of options (formed from a combination
+    '''Write the complete set of options (formed from a combination
     of command-line, environment variable, config file and defaults)
     to an output file.
+
+    Parameters
+    ----------
+    options : _type_
+        _description_
+    config_log_path : _type_
+        _description_
     '''
     with open(config_log_path, 'wt') as log:
         for option, value in options.__dict__.items():
@@ -393,17 +523,20 @@ def write_config_log(options, config_log_path):
 
 #-----------------------------------------------
 def get_function_args(fun, options):
-    '''
-    Iterate through attributes in a namepsace object and return attributes
+    '''Iterate through attributes in a namepsace object and return attributes
     included in the signature of the given function as a dictionary
 
-    Inputs:
-        fun: function object
+    Parameters
+    ----------
+    fun : function
+        function object
+    options : namespace
+        Simple namepsace object returned from argparse
 
-        options: Simple namepsace object returned from argparse
-
-    Outputs:
-        args_dict: dictionary of attribute/values from options where
+    Returns
+    -------
+    args_dict
+        dictionary of attribute/values from options where
         an attribute is included iff it is a member of fun's signature
     '''
     #Get list of arg names from the given function signature
@@ -421,18 +554,33 @@ def get_function_args(fun, options):
 
 #-----------------------------------------------
 def bool_option(v):
-    '''
-    Convert various string inputs that might be interpreted as yes/no, true/false to bool.
+    '''Convert various string inputs that might be interpreted as yes/no, true/false to bool.
     If input is already a bool, just return itself
+    
     Inputs:
         v : str
             variable to convert to bool
     Notes:
-        'yes', 'true', 't', 'y', '1' convert to True
+        
 
-        'no', 'false', 'f', 'n', '0' convert to False
+    Parameters
+    ----------
+    v : str
+        variable to convert to bool
+
+    Returns
+    -------
+    bool
+        True, 'yes', 'true', 't', 'y', '1' convert to True
+
+        False, 'no', 'false', 'f', 'n', '0' convert to False
 
         Any other input raises an argparse ArgumentTypeError
+
+    Raises
+    ------
+    configargparse.ArgumentTypeError
+        _description_
     '''
     if isinstance(v, bool):
        return v
@@ -441,6 +589,4 @@ def bool_option(v):
     elif str(v).lower() in ('no', 'false', 'f', 'n', '0'):
         return False
     else:
-        raise configargparse.ArgumentTypeError('Boolean value expected.')
-        
-
+        raise configargparse.ArgumentTypeError('Boolean value expected.')
\ No newline at end of file
